#!/usr/sbin/perl
#
# AncSeq.pl v1.0
# Last modified August 4, 2003 16:16
# (c) Olaf R.P. Bininda-Emonds
#
# Usage 'perl AncSeq.pl inFile'
#
# Creates random distributions of events based on characteristics of a Normal Table

$inFile = MacPerl::Ask("Enter name of PAUP* logfile","testdata.log3");
$NoCorrection = MacPerl::Answer("Correct for highly conflicting events?", "No", "Yes");
$verbosePick = MacPerl::Answer("Verbose output?", "No", "Yes");
	$verbose = 1 if ($verbosePick == 0);

($outFile) = split(/\./,$inFile,0);
$outFile .= ".anc";

open (fIN,"<$inFile");
#open (fOUT,">$outFile");
#	print fOUT "Results based on $maxRep replicates\n";
#	print fOUT "Species\tEvents\tStages\tUsed Stages\tMean SimUsed\tSD SimUsed\tZ SimUsed\tSimul EPs\tMean SimSimul\tSD SimSimul\tZ SimSimul\n";

$count = 0;

while (<fIN>)
	{
	print "$_";
	if (m/----------/)
		{
		$DataFlag = 1;	# Indicates that next line(s) will contain data
		}
	if ($_ =~ /^\n/)	# A completely blank line
		{
		$DataFlag = 0;	# Indicates end of data block
		}
	if ($_ =~ /^\d+\w/ and $DataFlag)
		{
		chop;
		($Node,$EPdata) = split;
		if (not $NodePresent{$Node})
			{
			$NodePresent{$Node} = 1;
			$NodeID{$Node} = $count;
			$NodeName{$count} = $Node;
			$DevelSeq[$NodeID{$Node}] = $EPdata;
			$count++;
			}
		else
			{
			$DevelSeq[$NodeID{$Node}] .= $EPdata;			
			}
		}
	}

$NumEP = length($DevelSeq[0]);	# Derive number of events from event-pairs
$NumEvent = int(sqrt($NumEP*2));
until (($NumEvent*$NumEvent - $NumEvent) / 2 == $NumEP)
	{
	$NumEvent++;
	}
print "Event-pairs: $NumEP\tEvents: $NumEvent\n" if ($verbose);
$CharNum = 1;
for ($Row = 2; $Row <= $NumEvent; $Row++)	# Assign constituent events to each event-pair
	{
	for ($Column = 1; $Column < $Row; $Column++)
		{
		$ColumnEvent{$CharNum} = $Column;
		$RowEvent{$CharNum} = $Row;
		$CharNum++
		}
	}

$count = 0;
foreach $seq (@DevelSeq)
	{
	@score = 0;
	for ($offset = 1; $offset <= $NumEP; $offset++)
		{
		$EP[$ColumnEvent{$offset}][$RowEvent{$offset}] = substr($seq,$offset-1,1);
		print "Num: $offset\tEP: $EP[$ColumnEvent{$offset}][$RowEvent{$offset}]\tColumn: $ColumnEvent{$offset}\tRow: $RowEvent{$offset}\n" if ($verbose);
		if ($EP[$ColumnEvent{$offset}][$RowEvent{$offset}] == 0)
			{
			$score[$RowEvent{$offset}]--;
			$score[$ColumnEvent{$offset}]++;
			}
		if ($EP[$ColumnEvent{$offset}][$RowEvent{$offset}] == 2)
			{
			$score[$RowEvent{$offset}]++;
			$score[$ColumnEvent{$offset}]--;
			}
		}
	for (1..$NumEvent)
		{
		print "Event: $_\tScore: $score[$_]\n" if ($verbose);
		}
#	@EventSort = sort {$a <=> $b} @score;
#	@score = (1,1,1,1,1);
#	print "@score\n";

	for (1..$NumEvent)	#Subroutine to generate index and rank tables
		{
		$preIT{$_} = $score[$_];
		}
	@indexTable = sort {$preIT{$a} <=> $preIT{$b};} keys %preIT;
	for (1..$NumEvent)
		{
		$SeqRankTable[$indexTable[$_-1]] = $_;
#		$EventRankTable[$indexTable[$_]] = $_+1-$matchOne;	#Correction to give tied ranks the same rank without incrementing rank
#		$matchOne++ if ($Events[$indexTable[$_]-1] == $Events[$indexTable[$_+1]-1]);
		}

	print "@SeqRankTable\n";

	for (1..$NumEvent)	# Subroutine to correct for tied ranks
		{
		$rankList{$SeqRankTable[$_]} = $score[$_];
		}
	
	$element = $NumEvent;
	while ($element > 1)
		{
		$match = 1;
		print "Element: $element\tMatch: $match\n";
		if ($rankList{$element} == $rankList{$element-1})
			{
			$topRank = $element;
			$RankSum = $element;
			until ($rankList{$element} != $rankList{$element-$match} or $element-$match == 0)
				{
				$RankSum += ($element-$match);
				$match++;
				}
			$TiedRank = $RankSum / $match;
			$fix = 1;

			for (1..$NumEvent)
				{
				for ($topRank-$match+1..$topRank)
					{
					if ($SeqRankTable[$fix] == $_)
						{
						$SeqRankTable[$fix] = $TiedRank;
						}
					}
				$fix++;
				}
			}
		$element -= $match;
		}
	
	print "@SeqRankTable\n";
	
	for ($Event = 1; $Event <= $NumEvent; $Event++)
		{
		$Max[$Event] = $Min[$Event] = $SeqRankTable[$Event];
		$conflict[$Event] = 0;
		print "Event: $Event\n" if ($verbose);
		for ($EventComp = 1; $EventComp <= $NumEvent; $EventComp++)
			{
			unless ($Event == $EventComp)
				{
				$direction = $SeqRankTable[$Event] - $SeqRankTable[$EventComp];
				if ($Event < $EventComp)
					{
					$EventPair = $EP[$Event][$EventComp];
					}
				else
					{
					$InverseEP = $EP[$EventComp][$Event];
					$EventPair = 2 - $InverseEP;
					}
				print "\tCompared event: $EventComp\tDirection: $direction\tEP: $EventPair\n";
				if ($direction < 0 and $EventPair == 1)
					{
					$Max[$Event] = $SeqRankTable[$EventComp] if ($SeqRankTable[$EventComp] > $Max[$Event]);
					$conflict[$Event]++;
					print "Conflict 1!\n" if ($verbose);
					}
				if ($direction < 0 and $EventPair == 0)
					{
					$Max[$Event] = $SeqRankTable[$EventComp] + 0.5 if ($SeqRankTable[$EventComp] > $Max[$Event]);
					$conflict[$Event]++;
					print "Conflict 2!\n" if ($verbose);
					}
				if ($direction == 0 and $EventPair == 0)
					{
					$Max[$Event] = $SeqRankTable[$EventComp] + 0.5 if ($SeqRankTable[$EventComp] > $Max[$Event]);
					$conflict[$Event]++;
					print "Conflict 3!\n" if ($verbose);
					}
				if ($direction == 0 and $EventPair == 2)
					{
					$Min[$Event] = $SeqRankTable[$EventComp] - 0.5 if ($SeqRankTable[$EventComp] < $Min[$Event]);
					$conflict[$Event]++;
					print "Conflict 4!\n" if ($verbose);
					}
				if ($direction > 0 and $EventPair == 1)
					{
					$Min[$Event] = $SeqRankTable[$EventComp] if ($SeqRankTable[$EventComp] < $Min[$Event]);
					$conflict[$Event]++;
					print "Conflict 5!\n" if ($verbose);
					}
				if ($direction > 0 and $EventPair == 2)
					{
					$Min[$Event] = $SeqRankTable[$EventComp] - 0.5 if ($SeqRankTable[$EventComp] < $Min[$Event]);
					$conflict[$Event]++;
					print "Conflict 6!\n" if ($verbose);
					}
				}
			}
		}
	
	@ConflictSort = sort {$a <=> $b} @conflict;
	if (int($NumEvent/2)*2 == $NumEvent)	# Even number of events
		{
		$MedianConflict = ($ConflictSort[$NumEvent/2] + $ConflictSort[($NumEvent/2)+1]) / 2;
		}
	else
		{
		$MedianConflict = $ConflictSort[($NumEvent/2)+0.5];
		}

	for ($Event = 1; $Event <= $NumEvent; $Event++)
		{
		for ($EventComp = 1; $EventComp <= $NumEvent; $EventComp++)
			{
			unless ($Event == $EventComp or $conflict[$EventComp] > $MedianConflict)
				{
				$direction = $SeqRankTable[$Event] - $SeqRankTable[$EventComp];
				if ($Event < $EventComp)
					{
					$EventPair = $EP[$Event][$EventComp];
					}
				else
					{
					$InverseEP = $EP[$EventComp][$Event];
					$EventPair = 2 - $InverseEP;
					}
				if ($direction < 0 and $EventPair == 1)
					{
					$MaxCorr[$Event] = $SeqRankTable[$EventComp] if ($SeqRankTable[$EventComp] > $MaxCorr[$Event]);
					print "Conflict 1!\n" if ($verbose);
					}
				if ($direction < 0 and $EventPair == 0)
					{
					$MaxCorr[$Event] = $SeqRankTable[$EventComp] + 0.5 if ($SeqRankTable[$EventComp] > $MaxCorr[$Event]);
					print "Conflict 2!\n" if ($verbose);
					}
				if ($direction == 0 and $EventPair == 0)
					{
					$MaxCorr[$Event] = $SeqRankTable[$EventComp] + 0.5 if ($SeqRankTable[$EventComp] > $MaxCorr[$Event]);
					print "Conflict 3!\n" if ($verbose);
					}
				if ($direction == 0 and $EventPair == 2)
					{
					$MinCorr[$Event] = $SeqRankTable[$EventComp] - 0.5 if ($SeqRankTable[$EventComp] < $MinCorr[$Event]);
					print "Conflict 4!\n" if ($verbose);
					}
				if ($direction > 0 and $EventPair == 1)
					{
					$MinCorr[$Event] = $SeqRankTable[$EventComp] if ($SeqRankTable[$EventComp] < $MinCorr[$Event]);
					print "Conflict 5!\n" if ($verbose);
					}
				if ($direction > 0 and $EventPair == 2)
					{
					$MinCorr[$Event] = $SeqRankTable[$EventComp] - 0.5 if ($SeqRankTable[$EventComp] < $MinCorr[$Event]);
					print "Conflict 6!\n" if ($verbose);
					}
				}
			}
		}

	print "Node: $NodeName{$count}\n" if ($verbose);
		$count++;
	for ($Event = 1; $Event <= $NumEvent; $Event++)
		{
		print "Event: $Event\tRank: $SeqRankTable[$Event]\tMin: $Min[$Event]\tMax: $Max[$Event]\tConflict: $conflict[$Event]\n" if ($verbose);
		}

		
	}

close fIN;
#close fOUT;
	MacPerl::SetFileInfo("XCEL", "TEXT", $outFile);

